import importlib
import json
import logging
import uuid
from typing import Tuple, Callable, Dict, Any, Union

import pika
from django.utils import timezone



class VirtualHost(object):
    """
    A RabbitMQ virtual host
    """

    def __init__(self,
                 url: str = None,
                 host: str = 'localhost',
                 name: str = '%2f',
                 port: int = 5672,
                 username: str = 'guest',
                 password: str = 'guest',
                 secure: bool = False) -> None:

        self.secure = secure
        if not url:
            self.host = host
            self.name = name
            self.port = port
            self.username = username
            self.password = password

        else:
            try:
                url = url.replace('amqp://', '')
                if '@' in url:
                    # user:pass@host:port/vhost
                    credentials, url = url.split('@')
                    self.username, self.password = credentials.split(':')

                else:
                    # host:port/vhost
                    self.username = username
                    self.password = password
                import pika

                try:
                    url, self.name = url.split('/')
                except ValueError:
                    url = url.split('/')[0]
                    self.name = name

                self.host, _port = url.split(':')
                self.port = int(_port)

            except Exception as err:
                raise Exception('Unable to parse the RabbitMQ server. Please check your configuration: %s' % err)

            if not self.name:
                self.name = '%2f'

    def __str__(self) -> str:
        """
        Returns the broker url
        """
        return 'amqp://%s:%s@%s:%s/%s' % (self.username, self.password, self.host, self.port, self.name)

    @property
    def blocking_connection(self) -> pika.BlockingConnection:
        """
        Connect to the VHOST
        """
        credentials = pika.PlainCredentials(username=self.username, password=self.password)
        if self.name == '%2f':
            vhost = '/'
        else:
            vhost = self.name

        params = pika.ConnectionParameters(host=self.host, port=self.port, virtual_host=vhost,
                                           credentials=credentials, connection_attempts=10, ssl=self.secure,
                                           heartbeat=1200)
        return pika.BlockingConnection(parameters=params)


class BaseMessageSerializer(object):
    """
    A class that defines how to convert a RabbitMQ message into an executable python function from your Django project,
    and back again

    :param Message message: the RabbitMQ message

    """
    content_type = 'application/json'
    type_header, message_type = ('',) * 2
    task_get_attempts = 20

    @classmethod
    def get_task(cls, properties: pika.BasicProperties, body: bytes) -> Callable:
        """
        Identifies the python function to be executed from the content of the RabbitMQ message. By default, Carrot
        returns the value of the self.type_header header in the properties.

        Once this string has been found, carrot uses importlib to return a callable python function.

        """
        mod = '.'.join(properties.headers[cls.type_header].split('.')[:-1])
        task = properties.headers[cls.type_header].split('.')[-1]
        module = importlib.import_module(mod)
        func = getattr(module, task)
        return func

    def __init__(self, message: 'Message') -> None:
        self.message = message

    def publish_kwargs(self) -> dict:
        """
        Returns a dictionary of keyword arguments to be passed to channel.basic_publish. In this implementation, the
        exchange, routing key and message body are returned

        """
        exchange = self.message.exchange or ''
        routing_key = self.message.routing_key or 'default'
        body = self.body()
        return dict(exchange=exchange, routing_key=routing_key, body=body, mandatory=True)

    def body(self) -> str:
        """
        Returns the content to be added to the RabbitMQ message body

        By default, this implementation returns a simple dict in the following format:

        .. code-block:: python

            {
                'args': ('tuple', 'of', 'positional', 'arguments'),
                'kwargs': {
                    'keyword1': 'value',
                    'keyword2': True
                }
            }

        """
        args = self.message.task_args
        kwargs = self.message.task_kwargs

        data: Dict[str, Any] = {}

        if args:
            data['args'] = args

        if kwargs:
            data['kwargs'] = kwargs

        return json.dumps(data)

    def properties(self) -> dict:
        """
        Returns a dict from which a :class:`pika.BasicProperties` object can be created

        In this implementation, the following is returned:
        - headers
        - content message_type
        - priority
        - message id
        - message message_type

        """
        headers = {self.type_header: self.message.task}
        content_type = self.content_type
        priority = self.message.priority
        message_id = str(self.message.uuid)
        message_type = self.message_type

        return dict(headers=headers, content_type=content_type, priority=priority, message_id=message_id,
                    type=message_type)

    def publish(self, connection: pika.BlockingConnection, channel: pika.channel.Channel) -> None:
        """
        Publishes a message to the channel
        """
        kwargs = self.publish_kwargs()
        kwargs['properties'] = pika.BasicProperties(**self.properties())
        channel.basic_publish(**kwargs)
        connection.close()

    @classmethod
    def serialize_arguments(cls, body: str) -> Tuple[tuple, dict]:
        """
        Extracts positional and keyword arguments to be sent to a function from the message body
        """
        content = json.loads(body)
        args = content.get('args', ())
        kwargs = content.get('kwargs', {})
        return args, kwargs


class DefaultMessageSerializer(BaseMessageSerializer):
    type_header = 'type'
    message_type = 'django-carrot message'


# noinspection PyUnresolvedReferences
class Message(object):
    """
    A message to publish to RabbitMQ. Takes the following parameters:

    .. note::
        Your RabbitMQ queue must support message priority for the *priority* parameter to have any affect. You need to
        define the x-max-priority header when creating your RabbitMQ queue to do this. See
        `Priority Queue Support <https://www.rabbitmq.com/priority.html>`_ for more details. Carrot applies a maximum
        priority of **255** by default to all queues it creates automatically.

    .. warning::
        You should not attempt to create instances of this object yourself. You should use the
        :func:`carrot.utilities.create_msg` function instead

    """

    def __init__(self,
                 task: str,
                 virtual_host: VirtualHost = None,
                 queue: str = 'default',
                 routing_key: str = None,
                 exchange: str = '',
                 priority: int = 0,
                 task_args: tuple = (),
                 task_kwargs: Union[str, dict] = None) -> None:

        if not task_kwargs or task_kwargs in ['{}', '"{}"']:
            task_kwargs = {}

        if not routing_key:
            routing_key = queue

        if not virtual_host:
            from carrot.utilities import get_host_from_name
            virtual_host = get_host_from_name(queue)

        assert isinstance(virtual_host, VirtualHost)

        self.uuid = str(uuid.uuid4())

        self.virtual_host = virtual_host
        self.exchange = exchange
        self.queue = queue
        self.routing_key = routing_key
        self.priority = priority

        self.task = task
        self.task_args = task_args
        self.task_kwargs = task_kwargs

        self.formatter = DefaultMessageSerializer(self)

    @property
    def connection_channel(self) -> Tuple[pika.BlockingConnection, pika.channel.Channel]:
        """
        Gets or creates the queue, and returns a tuple containing the object's VirtualHost's blocking connection,
        and its channel
        """
        connection = self.virtual_host.blocking_connection
        channel = connection.channel()

        return connection, channel

    def publish(self, pika_log_level: int = logging.ERROR) -> Any:
        """
        Publishes the message to RabbitMQ queue and creates a MessageLog object so the progress of the task can be
        tracked in the Django project's database
        """
        from carrot.models import MessageLog
        logging.getLogger("pika").setLevel(pika_log_level)
        connection, channel = self.connection_channel

        if isinstance(self.task_kwargs, str):
            try:
                json.dumps(self.task_kwargs)
                keyword_arguments = self.task_kwargs
            except json.decoder.JSONDecodeError:
                keyword_arguments = '{}'
        else:
            keyword_arguments = json.dumps(self.task_kwargs)

        log = MessageLog.objects.create(
            status='PUBLISHED',
            queue=self.queue,
            exchange=self.exchange or '',
            routing_key=self.routing_key or self.queue,
            uuid=str(self.uuid),
            priority=self.priority,
            task_args=self.task_args,
            content=keyword_arguments,
            task=self.task,
            publish_time=timezone.now(),
        )

        self.formatter.publish(connection, channel)
        return log



from django.conf.urls import url
from carrot.views import MessageList
from carrot.utilities import decorate_class_view, decorate_function_view, create_class_view
from django.conf import settings
from carrot.api import (
    published_message_log_viewset, failed_message_log_viewset, completed_message_log_viewset, scheduled_task_viewset,
    detail_message_log_viewset, scheduled_task_detail, run_scheduled_task, task_list, validate_args, purge_messages,
    MessageLogViewset, requeue_pending
)
from typing import Any

try:
    decorators = settings.CARROT.get('monitor_authentication', [])
except AttributeError:
    decorators = []


def _(v: Any, **kwargs) -> Any:
    """
    Decorates a class based view with a custom auth decorator specified in the settings module
    """
    return decorate_class_view(v, decorators).as_view(**kwargs)


def _f(v: MessageLogViewset) -> Any:
    """
    The same as the above _ method, but for function-based views
    """
    return decorate_function_view(v, decorators)


urlpatterns = [
    url(r'^$', _(MessageList), name='carrot-monitor'),
    url(r'^api/message-logs/published/$', _f(published_message_log_viewset), name='published-messagelog'),
    url(r'^api/message-logs/failed/$', _f(failed_message_log_viewset)),
    url(r'^api/message-logs/purge/$', _f(purge_messages)),
    url(r'^api/message-logs/requeue/$', _f(requeue_pending)),
    url(r'^api/message-logs/completed/$', _f(completed_message_log_viewset)),
    url(r'^api/message-logs/(?P<pk>[0-9]+)/$', _f(detail_message_log_viewset)),
    url(r'^api/scheduled-tasks/$', _f(scheduled_task_viewset)),
    url(r'^api/scheduled-tasks/task-choices/$', _f(task_list)),
    url(r'^api/scheduled-tasks/validate-args/$', _f(validate_args)),
    url(r'^api/scheduled-tasks/(?P<pk>[0-9]+)/$', _f(scheduled_task_detail)),
    url(r'^api/scheduled-tasks/(?P<pk>[0-9]+)/run/$', _f(run_scheduled_task)),
]



"""
This module contains a number of helper functions for performing basic Carrot functions, e.g. publish, schedule and
consume

Most users should use the functions defined in this module, rather than attempting to subclass the base level objects

"""
import json
import importlib
from django.conf import settings
from carrot.objects import VirtualHost, Message
from carrot.models import ScheduledTask, MessageLog
from django.utils.decorators import method_decorator
from carrot import DEFAULT_BROKER
from carrot.exceptions import CarrotConfigException
from django.db.utils import IntegrityError
from typing import Dict, List, Union, Callable, Type, Any


def get_host_from_name(name: str) -> VirtualHost:
    """
    Gets a host object from a given queue name based on the Django configuration

    If no queue name is provided (as may be the case from some callers), this function returns a VirtualHost based on
    the CARROT.default_broker value.

    May raise an exception if the given queue name is not registered in the settings.
    """
    try:
        carrot_settings = settings.CARROT
    except AttributeError:
        carrot_settings = {
            'default_broker': DEFAULT_BROKER,
            'queues'        : [
                {
                    'name': 'default',
                    'host': DEFAULT_BROKER
                }
            ]
        }

    try:
        if not name:
            try:
                conf = carrot_settings.get('default_broker', {})
            except AttributeError:
                conf = {}

            if not conf:
                conf = {'url': DEFAULT_BROKER}
            elif isinstance(conf, str):
                conf = {'url': conf}

            return VirtualHost(**conf)

        queues = carrot_settings.get('queues', [])
        queue_host = list(filter(lambda queue: queue['name'] == name, queues))[0]['host']
        try:
            vhost = VirtualHost(**queue_host)
        except TypeError:
            vhost = VirtualHost(url=queue_host)

        return vhost

    except IndexError:
        raise CarrotConfigException('Cannot find queue called %s in settings.CARROT queue list' % name)


def validate_task(task: Union[str, Callable]) -> str:
    """
    Helper function for dealing with task inputs which may either be a callable, or a path to a callable as a string

    In case of a string being provided, this function checks whether the import path leads to a valid callable

    Otherwise, the callable is converted back into a string (as the :class:`carrot.objects.Message` requires a string
    input)

    This function is used by the following other utility functions:
    - :func:`.create_scheduled_task`
    - :func:`.create_message`

    """
    mod, fname = (None,) * 2

    if isinstance(task, str):
        try:
            fname = task.split('.')[-1]
            mod = '.'.join(task.split('.')[:-1])
            module = importlib.import_module(mod)
            getattr(module, fname)
        except ImportError as err:
            raise ImportError('Unable to find the module: %s' % err)

        except AttributeError as err:
            raise AttributeError('Unable to find a function called %s in the module %s: %s' % (fname, mod, err))
    else:
        # noinspection PyUnresolvedReferences
        task = '%s.%s' % (task.__module__, task.__name__)

    return task


def create_message(task: Union[str, Callable],
                   queue: str,
                   priority: int = 0,
                   task_args: tuple = (),
                   exchange: str = '',
                   routing_key: str = None,
                   task_kwargs: dict = None
                   ) -> Message:
    """
    Creates a :class:`carrot.objects.Message` object without publishing it

    The task to execute (as a string or a callable) needs to be supplied. All other arguments are optional
    """

    if not task_kwargs:
        task_kwargs = {}

    task = validate_task(task)

    vhost = get_host_from_name(queue)
    msg = Message(virtual_host=vhost, queue=queue, routing_key=routing_key, exchange=exchange, task=task,
                  priority=priority, task_args=task_args, task_kwargs=task_kwargs)

    return msg


def publish_message(task: Union[str, Callable],
                    *task_args,
                    priority: int = 0,
                    queue: str = None,
                    exchange: str = '',
                    routing_key: str = None,
                    **task_kwargs) -> MessageLog:
    """
    Wrapped for :func:`.create_message`, which publishes the task to the queue

    This function is the primary method of publishing tasks to a message queue
    """
    if not queue:
        queue = 'default'
    msg = create_message(task, queue, priority, task_args, exchange, routing_key, task_kwargs)
    return msg.publish()


def create_scheduled_task(task: Union[str, Callable],
                          interval: Dict[str, int],
                          task_name: str = None,
                          queue: str = None,
                          **kwargs) -> ScheduledTask:
    """
    Helper function for creating a :class:`carrot.models.ScheduledTask`
    """

    if not task_name:
        if isinstance(task, str):
            task_name = task
        else:
            raise Exception('You must provide a task_name or task')

    task = validate_task(task)

    try:
        assert isinstance(interval, dict)
        assert len(interval.items()) == 1
    except AssertionError:
        raise AttributeError('Interval must be a dict with a single key value pairing, e.g.: {\'seconds\': 5}')

    interval_type, count = list(*interval.items())

    try:
        t = ScheduledTask.objects.create(
                queue=queue,
                task_name=task_name,
                interval_type=interval_type,
                interval_count=count,
                routing_key=queue,
                task=task,
                content=json.dumps(kwargs or '{}'),
        )
    except IntegrityError:
        raise IntegrityError('A ScheduledTask with this task_name already exists. Please specific a unique name using '
                             'the task_name parameter')

    return t


def get_mixin(decorator: Callable) -> Type[object]:
    """
    Helper function that allows dynamic application of decorators to a class-based views

    :param func decorator: the decorator to apply to the view
    """

    class Mixin(object):
        @method_decorator(decorator)
        def dispatch(self, request, *args, **kwargs):
            return super(Mixin, self).dispatch(request, *args, **kwargs)

    return Mixin


def create_class_view(view: Any, decorator: Any) -> object:
    """
    Applies a decorator to the dispatch method of a given class based view. Can be chained
    """
    mixin: Any = get_mixin(decorator)
    base_view: Any = view

    class DecoratedView(mixin, base_view):
        pass

    return DecoratedView


def decorate_class_view(view_class: object,
                        decorators: List[str] = None) -> Any:
    """
    Loop through a list of string paths to decorator functions, and call :func:`.create_class_view` for each one
    """
    if decorators is None:
        decorators = []

    for decorator in decorators:
        _module = '.'.join(decorator.split('.')[:-1])
        module = importlib.import_module(_module)
        _decorator = getattr(module, decorator.split('.')[-1])
        view_class = create_class_view(view_class, _decorator)

    return view_class


def create_function_view(view: Callable, decorator: Callable) -> Callable:
    """
    Similar to :func:`.create_class_view`, but attaches a decorator to a function based view, instead of a class-based
    one
    """

    @decorator
    def wrap(request, *args, **kwargs):
        return view(request, *args, **kwargs)

    return wrap


def decorate_function_view(view: Any, decorators: List[str] = None) -> Any:
    """
    Similar to :func:`.decorate_class_view`, but for function based views
    """
    if not decorators:
        decorators = []

    for decorator in decorators:
        _module = '.'.join(decorator.split('.')[:-1])
        module = importlib.import_module(_module)
        _decorator = getattr(module, decorator.split('.')[-1])
        view = create_function_view(view, _decorator)

    return view


# noinspection PyUnresolvedReferences
def purge_queue() -> None:
    """
    Deletes all MessageLog objects with status `IN_PROGRESS` or `PUBLISHED` add iterate through and purge all RabbitMQ
    queues
    """
    queued_messages = MessageLog.objects.filter(status__in=['IN_PROGRESS', 'PUBLISHED'])
    queued_messages.delete()

    try:
        carrot_settings = settings.CARROT
    except AttributeError:
        carrot_settings = {
            'default_broker': DEFAULT_BROKER,
        }

    queues = carrot_settings.get('queues', [{'name': 'default', 'host': DEFAULT_BROKER}])
    for queue in queues:
        if type(queue['host']) is str:
            filters = {'url': queue['host']}
        else:
            filters = queue['host']
        host = VirtualHost(**filters)
        channel = host.blocking_connection.channel()
        channel.queue_purge(queue=queue['name'])


def requeue_all() -> None:
    """
    Requeues all pending MessageLogs
    """
    logs = MessageLog.objects.filter(status__in=['IN_PROGRESS', 'PUBLISHED'])

    for log in logs:
        log.requeue()



import threading
import time
from typing import List

from django.core.exceptions import ObjectDoesNotExist

from carrot.models import ScheduledTask


class ScheduledTaskThread(threading.Thread):
    """
    A thread that handles a single :class:`carrot.models.ScheduledTask` object. When started, it waits for the interval
    to pass before publishing the task to the required queue

    While waiting for the task to be due for publication, the process continuously monitors the object in the Django
    project's database for changes to the interval, task, or arguments, or in case it gets deleted/marked as inactive
    and response accordingly
    """

    def __init__(self,
                 scheduled_task: ScheduledTask,
                 run_now: bool = False,
                 **filters) -> None:
        threading.Thread.__init__(self)
        self.id = scheduled_task.pk
        self.queue = scheduled_task.routing_key
        self.scheduled_task = scheduled_task
        self.run_now = run_now
        self.active = True
        self.filters = filters
        self.inactive_reason = ''

    def run(self) -> None:
        """
        Initiates a timer, then once the timer is equal to the ScheduledTask's interval, the scheduler
        checks to make sure that the task has not been deactivated/deleted in the mean time, and that the manager
        has not been stopped, then publishes it to  the queue
        """
        interval = self.scheduled_task.multiplier * self.scheduled_task.interval_count

        count = 0
        if self.run_now:
            self.scheduled_task.publish()

        while True:
            while count < interval:
                if not self.active:
                    if self.inactive_reason:
                        print('Thread stop has been requested because of the following reason: %s.\n Stopping the '
                              'thread' % self.inactive_reason)

                    return

                try:
                    self.scheduled_task = ScheduledTask.objects.get(pk=self.scheduled_task.pk, **self.filters)
                    interval = self.scheduled_task.multiplier * self.scheduled_task.interval_count

                except ObjectDoesNotExist:
                    print('Current task has been removed from the queryset. Stopping the thread')
                    return

                time.sleep(1)
                count += 1

            print('Publishing message %s' % self.scheduled_task.task)
            self.scheduled_task.publish()
            count = 0


class ScheduledTaskManager(object):
    """
    The main scheduled task manager project. For every active :class:`carrot.models.ScheduledTask`, a
    :class:`ScheduledTaskThread` is created and started

    This object exists for the purposes of starting these threads on startup, or when a new ScheduledTask object
    gets created, and implements a .stop() method to stop all threads

    """

    def __init__(self, **options) -> None:
        self.threads: List[ScheduledTaskThread] = []
        self.filters = options.pop('filters', {'active': True})
        self.run_now = options.pop('run_now', False)
        self.tasks = ScheduledTask.objects.filter(**self.filters)

    def start(self) -> None:
        """
        Initiates and starts a scheduler for each given ScheduledTask
        """
        print('found %i scheduled tasks to run' % self.tasks.count())
        for t in self.tasks:
            print('starting thread for task %s' % t.task)
            thread = ScheduledTaskThread(t, self.run_now, **self.filters)
            thread.start()
            self.threads.append(thread)

    def add_task(self, task: ScheduledTask) -> None:
        """
        After the manager has been started, this function can be used to add an additional ScheduledTask starts a
        scheduler for it
        """
        thread = ScheduledTaskThread(task, self.run_now, **self.filters)
        thread.start()
        self.threads.append(thread)

    def stop(self) -> None:
        """
        Safely stop the manager
        """
        print('Attempting to stop %i running threads' % len(self.threads))

        for t in self.threads:
            print('Stopping thread %s' % t)
            t.active = False
            t.inactive_reason = 'A termination of service was requested'
            t.join()
            print('thread %s stopped' % t)



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['carrot',
 'carrot.management',
 'carrot.management.commands',
 'carrot.migrations']

package_data = \
{'': ['*'], 'carrot': ['static/carrot/*', 'templates/carrot/*']}

install_requires = \
['django>=2.1,<3.0',
 'djangorestframework>=3.9,<4.0',
 'json2html>=1.2,<2.0',
 'pika>=0.13.1,<0.14.0',
 'psutil>=5.6,<6.0',
 'sphinx_bootstrap_theme>=0.6.5,<0.7.0']

setup_kwargs = {
    'name': 'django-carrot',
    'version': '1.4.4a0',
    'description': 'A RabbitMQ asynchronous task queue for Django.',
    'long_description': ".. image:: https://coveralls.io/repos/github/chris104957/django-carrot/badge.svg?branch=master\n    :target: https://coveralls.io/github/chris104957/django-carrot?branch=master\n\n.. image:: https://readthedocs.org/projects/django-carrot/badge/?version=latest\n    :target: http://django-carrot.readthedocs.io/en/latest/?badge=\n\n.. image:: https://travis-ci.org/chris104957/django-carrot.svg?branch=master\n    :target: https://travis-ci.org/chris104957/django-carrot.svg?branch=master\n\n.. image:: https://coveralls.io/repos/github/chris104957/django-carrot/badge.svg?branch=master\n    :target: https://coveralls.io/github/chris104957/django-carrot?branch=master)\n\n.. image:: https://badge.fury.io/py/django-carrot.svg\n    :target: https://badge.fury.io/py/django-carrot\n\n.. image:: https://img.shields.io/badge/License-Apache%202.0-blue.svg\n    :target: https://opensource.org/licenses/Apache-2.0\n\n.. image:: /docs/source/images/carrot-logo-big.png\n    :align: center\n\n**django-carrot** is a lightweight task queue backend for Django projects that uses the RabbitMQ message broker, with\nan emphasis on quick and easy configuration and task tracking\n\nInstallation\n------------\n\nInstall django-carrot:\n\n.. code-block:: bash\n\n    pip install django-carrot\n\nInstall and run RabbitMQ\n\n.. code-block:: bash\n\n    brew install rabbitmq\n    brew services start rabbitmq\n\nConfiguration\n-------------\n\n1. Add carrot to your Django project's settings module:\n\n.. code-block:: python\n\n    INSTALLED_APPS = [\n        ...\n        'carrot',\n        ...\n    ]\n\n\n2. Apply the carrot migrations to your project's database:\n\n.. code-block:: python\n\n    python manage.py migrate carrot\n\n\nUsage\n-----\n\nTo start the service:\n\n.. code-block:: bash\n\n    python manage.py carrot_daemon start\n\n\nTo run tasks asynchronously:\n\n.. code-block:: python\n\n    from carrot.utilities import publish_message\n\n    def my_task(**kwargs):\n        return 'hello world'\n\n    publish_message(my_task, hello=True)\n\n\n\nTo schedule tasks to run at a given interval\n\n.. code-block:: python\n\n    from carrot.utilities import create_scheduled_task\n\n    create_scheduled_task(my_task, {'seconds': 5}, hello=True)\n\n\n.. note::\n    The above commands must be made from within the Django environment\n\nDocker\n------\n\nA sample docker config is available `here <https://github.com/chris104957/django-carrot-docker>`_\n\nFull documentation\n------------------\n\nThe full documentation is available `here <https://django-carrot.readthedocs.io/>`_\n\nSupport\n-------\n\nIf you are having any issues, please `log an issue <https://github.com/chris104957/django-carrot/issues/new>`_\n\nContributing\n------------\n\nDjango-carrot uses `Packagr <https://www.packagr.app/>`_ to share development builds. If you'd like access to it,\nplease send me your email address at christopherdavies553@gmail.com so I can give you access\n\nLicense\n-------\n\nThe project is licensed under the Apache license.\n\nIcons made by Trinh Ho from `www.flaticon.com <www.flaticon.com>`_ is licensed by CC 3.0 BY\n",
    'author': 'Christoper Davies',
    'author_email': 'christopherdavies553@gmail.com',
    'url': 'https://django-carrot.readthedocs.io',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.3,<4.0.0',
}


setup(**setup_kwargs)



import json
import pprint
from django.core.management.base import BaseCommand, CommandError, CommandParser
from carrot.utilities import publish_message


class Command(BaseCommand):
    help = 'Queues a job for execution'

    def add_arguments(self, parser: CommandParser) -> None:
        parser.add_argument('job_name', type=str)
        parser.add_argument('job_args', type=str, nargs='+')

    def handle(self, *args, **options) -> None:
        job_name = options['job_name']
        job_args = options['job_args']

        if job_args:
            publish_message(job_name, *job_args)
        else:
            publish_message(job_name)
